#!/usr/bin/php
<?php
//error_reporting (E_ALL | E_STRICT);
//ini_set ('display_errors' , 1);

//------------- set necessary paramters -------------------------------------
if (strtoupper(substr(PHP_OS, 0, 3)) === 'WIN') {
    //WINDOWS user: set path to tools
    $convert='"C:\util\ImageMagick\convert.exe"';
    $escape='';

} else {
    //Unix/Mac: set path to tools here 
    $convert='/opt/ImageMagick/bin/convert';
    $escape='\\';
}

//extract 16 bit image to temp file
$rawimage='raw.hex';

//--------------------------------------------------------------------------

$shortopts  = "";
$shortopts .= "i:";  // Required value
$shortopts .= "o:";  
$shortopts .= "r:";  
$shortopts .= "h";  

$longopts  = array(
    "help",       
);
$options = getopt($shortopts, $longopts);
//var_dump($options);

function help()
{
global $argv;
echo <<<EOT

usage: $argv[0] [options] -i ir_file.jpg -r 16BitImage -o outputimage.jpg

Settings:
-i ir_file.jpg      flir radiometric image
-r 16BitImage.png   16 Bit Image with radiometric sensor values (png/tif)
-o output           save a rebuild flir radiometric image with new 16BitImage

Options Summary:
--help              print this help
  
# source: 
# [1] http://u88.n24.queensu.ca/exiftool/forum/index.php/topic,4898.0.html

EOT;
}

if (isset($options['help']) || isset($options['h']))
{
 help();
};

if (isset($options['i']))
{
    $flirimg=$options['i'];
    if (isset($options['o']))
    {
        $destimg=$options['o'];    
    } else {
      print 'Error: No output file specified! "-o filename"'."\n";
      exit(1);
    }
    if (isset($options['r']))
    {
        $raw16img=$options['r'];    
    } else {
      print 'Error: No 16 Bit image source file specified! "-r filename"'."\n";
      exit(1);
    }

} else {     
    help();
    exit(1);
};

function Get32u($string,$pos) {
  $m = unpack("N", substr($string,$pos,4));
  return $m[1];
}
function Get16u($string,$pos) {
  $m = unpack("n", substr($string,$pos,2));
  return $m[1];
}

//extract and cut FFF from radiometric jpg 
// split with explode (binary safe)
$delimiter="\x46\x4c\x49\x52\x00\x01";
// open the binary file and get the content
$data= file_get_contents($flirimg);
$frameArr = explode($delimiter, $data);

$frame = $frameArr[0];
$m = Get16u($frame,-2);
$fff="";
$cut1=substr($frame,0,-4);

for($i=1;$i<count($frameArr);$i++) {
  $frame = $frameArr[$i];
  //skip first 2 bytes (segment i of j) and skip last 4 bytes ( 2 Byte MagicByteAppSegment + 2 Byte SegementSize) 
  $part=substr($frame,2,$m-strlen($delimiter)-4);
  //echo  "len: ".strlen($part)."\n";
  $fff=$fff.$part;
  $cut2=substr($frame,$m-strlen($delimiter)-2);
  //echo "cut: ".strlen($cut2)."\n";
  //calc next SegmentSize
  $m = Get16u($frame,-2);
}

// write extracted FFF File
file_put_contents($filename.".fff",$fff, LOCK_EX);
echo $filename.".fff";

echo "rebuilt FFF from $i parts\n";

//----------------------------------------------------------------------
echo "split FFF file to segments\n";

//$pos = 0x40; // start of FLIR record directory
$pos = ord(substr($fff,0x1B,1));
$dirLen=strlen($fff);

for (;;) {
    if ($dirLen < $pos + 0x20) break;
    $recType = Get16u($fff, $pos);
    if ($recType == 0) break;
    
    $recLen = Get32u($fff, $pos + 0x10);
    $recPos = Get32u($fff, $pos + 0x0c);
    
    printf ("FLIR Record 0x%02x, offset 0x%04x, length 0x%04x\n", $recType, $recPos, $recLen);
    if ( ($recPos < $pos) || (($recPos + $recLen) > $dirLen)) {
        echo "Invalid FLIR record\n";
        break;
    }

    if ($recType == 0x20) {
      //CameraInfo , little endian
      $CameraInfo=substr($fff,$recPos,$recLen);
    }
    $pos += 0x20;
}

//build new (reduced) FFF with two segments "0x20 -> CameraInfo" and "0x01 -> RawData"

//Header
$fffred=str_repeat("\x00",0x0200);                   //create empty 0x0200 Byte Header
$fffred=substr_replace($fffred,"FFF\x00MTX",0x00,7); //Fileformat ID: FFF\x00MTX
$fffred=substr_replace($fffred,"\x64",0x17,1);       //File format version
$fffred=substr_replace($fffred,"\x40",0x1B,1);       //Pointer to indexes
$fffred=substr_replace($fffred,"\x0E",0x1F,1);       //Number of indexes 0x40+0x0e*0x20=>0x0200 Header Size
$fffred=substr_replace($fffred,"\x02",0x23,1);       //dwNextID

//write CameraInfo Segment
$fffCamInfo="\x00\x20\x00\x01\x00\x00\x00\x71";                       //8 Byte wMainType+wSubType+dwVersion
$fffCamInfo=$fffCamInfo."\x00\x00\x00\x01\x00\x00\x02\x00";           //8 Byte dwIndexID+dwDataPtr
$fffCamInfo=$fffCamInfo.pack("N",strlen($CameraInfo));                //4 Byte dwDataSize -> sum 20 Byte
$fffred=substr_replace($fffred,$fffCamInfo,0x40,strlen($fffCamInfo)); //tagFLIRFILEINDEX (32 Byte)

$size = getimagesize($raw16img);
$width=$size[0];
$higth=$size[1];
//write new image size
$CameraInfo=substr_replace($CameraInfo,pack("v",$width),0x02,2);
$CameraInfo=substr_replace($CameraInfo,pack("v",$higth),0x04,2);
$CameraInfo=substr_replace($CameraInfo,pack("v",$width-1),0x0C,2);
$CameraInfo=substr_replace($CameraInfo,pack("v",$higth-1),0x10,2);
$fffred=$fffred.$CameraInfo;                                          //Add CameraInfo Data Segment

//write RAW Data Segment
//convert 16 bit png/tiff back to raw
exec($convert." $raw16img -depth 16 -endian lsb gray:$rawimage");
$raw1= file_get_contents($rawimage);
echo "new image w/h=".$width."/".$higth."\n";

$CameraRaw=str_repeat("\x00",0x020);                   //build empty 32 Byte Header
$CameraRaw=substr_replace($CameraRaw,"\x02",0x00,1);   //magic byte for RAW Data
//write new image size and the crop windows size
$CameraRaw=substr_replace($CameraRaw,pack("v",$width),0x02,2);
$CameraRaw=substr_replace($CameraRaw,pack("v",$higth),0x04,2);
$CameraRaw=substr_replace($CameraRaw,pack("v",$width-1),0x0C,2);
$CameraRaw=substr_replace($CameraRaw,pack("v",$higth-1),0x10,2);
//append 16 Bit sensor RAW values 
$CameraRaw=$CameraRaw.$raw1;

//build RAW header
$fffRaw="\x00\x01\x00\x02\x00\x00\x00\x65";                          //8 Byte wMainType+wSubType+dwVersion
$fffRaw=$fffRaw."\x00\x00\x00\x01";                                  //4 Byte dwIndexID
$fffRaw=$fffRaw.pack("N",strlen($fffred));                           //4 Byte dwDataPtr
$fffRaw=$fffRaw.pack("N",strlen($CameraRaw));                        //4 Byte dwDataSize -> sum 20 Byte
$fffred=substr_replace($fffred,$fffRaw,0x60,strlen($fffCamInfo));    //tagFLIRFILEINDEX 32 Byte

$fffred=$fffred.$CameraRaw;                                          //Add CameraRaw Data Segment
//write all contents to file 
file_put_contents($destimg.".fff",$fffred, LOCK_EX);

//----------------------------------------------------------------------
//rebuild radiometric jpg from jpg+fff
//max container size 2^16-2(App-Tag \xff\xe1)-2(SegmentSize)-6(Header \x46\x4c\x49\x52\x00\x01)-2(Seg Of Seg) 
// = 65.536-12= 65.524 Byte
echo "write new radiometric jpg\n";
$fff=$fffred;
$count=0;
$segments=(int)(strlen($fff)/65524);
$fff1="";
for($i=0;$i<strlen($fff);$i=$i+65524) {
  $fffArray[$count]=substr($fff,$i,65524);
  //10 Byte Header
  $len=strlen($fffArray[$count])+10;
  $len1=(int)($len/256);
  $len2=$len%256;
  echo "part ".$count.": ".$len." Byte \n";
  $fff1=$fff1."\xff\xe1".chr($len1).chr($len2)."\x46\x4c\x49\x52\x00\x01".chr($count).chr($segments).$fffArray[$count];
  $count++;
}
file_put_contents($destimg.".jpg",$cut1.$fff1.$cut2, LOCK_EX);
?> 